@@ -1,5 +1,5 @@
 package Mail::DMARC::Report::Store::SQL;
-our $VERSION = '1.20191025';
+our $VERSION = '1.20240314';
 use strict;
 use warnings;
 
@@ -84,7 +84,7 @@ sub next_todo {
     my ( $self ) = @_;
 
     if ( ! exists $self->{ _todo_list } ) {
-        $self->{_todo_list} = $self->query( $self->grammar->select_todo_query, [ time ] );
+        $self->{_todo_list} = $self->query( $self->grammar->select_todo_query, [ $self->time ] );
         return if ! $self->{_todo_list};
     }
 
@@ -110,8 +110,7 @@ sub retrieve_todo {
 
     # this method extracts the data from the SQL tables and populates a
     # list of Aggregate report objects with them.
-    my $reports = $self->query( $self->grammar->select_todo_query, [ time ] );
-
+    my $reports = $self->query( $self->grammar->select_todo_query, [ $self->time ] );
     my @reports_todo;
     return \@reports_todo if ! scalar @$reports;
 
@@ -136,18 +135,20 @@ sub delete_report {
     print "deleting report $report_id\n" if $self->verbose;
 
     # deletes with FK don't cascade in SQLite? Clean each table manually
-    my $rows = $self->query( $self->grammar->report_record_id,
-        [$report_id] );
-    my $row_ids = join( ',', map { $_->{id} } @$rows ) or return 1;
-    foreach my $table (
-        qw/ report_record_spf report_record_dkim report_record_reason /)
-    {
-        print "deleting $table rows $row_ids\n" if $self->verbose;
-        $self->query(
-            $self->grammar->delete_from_where_record_in($table, $row_ids));
+    my $rows = $self->query( $self->grammar->report_record_id, [$report_id] );
+    my @row_ids = map { $_->{id} } @$rows;
+
+    if (scalar @row_ids) {
+        foreach my $table (qw/ report_record_spf report_record_dkim report_record_reason /) {
+            print "deleting $table rows " . join(',', @row_ids) . "\n" if $self->verbose;
+            eval { $self->query( $self->grammar->delete_from_where_record_in($table), \@row_ids); };
+            # warn $@ if $@;
+        }
     }
     foreach my $table (qw/ report_policy_published report_record report_error /) {
-        $self->query( $self->grammar->delete_from_where_report($table), [$report_id] );
+        print "deleting $table rows for report $report_id\n" if $self->verbose;
+        eval { $self->query( $self->grammar->delete_from_where_report($table), [$report_id] ); };
+        # warn $@ if $@;
     }
 
     # In MySQL, where FK constraints DO cascade, this is the only query needed
@@ -205,7 +206,7 @@ sub get_report_id {
     # They aggregate on the From domain, where the DMARC policy was discovered
         $ids = $self->query(
         $self->grammar->select_id_with_end,
-        [ $from_dom_id, time, $author_id ]
+        [ $from_dom_id, $self->time, $author_id ]
         );
     };
 
@@ -431,7 +432,7 @@ sub insert_agg_record {
 sub insert_error {
     my ( $self, $rid, $error ) = @_;
     # wait >5m before trying to deliver this report again
-    $self->query($self->grammar->insert_error(0), [time + (5*60), $rid]);
+    $self->query($self->grammar->insert_error(0), [$self->time + (5*60), $rid]);
 
     return $self->query(
         $self->grammar->insert_error(1),
@@ -451,7 +452,7 @@ sub insert_rr_dkim {
     my ( $self, $row_id, $dkim ) = @_;
     my (@fields, @values);
     foreach ( qw/ domain selector result human_result / ) {
-        next if ! $dkim->{$_};
+        next if ! defined $dkim->{$_};
         if ( 'domain' eq $_ ) {
             push @fields, 'domain_id';
             push @values, $self->get_domain_id( $dkim->{domain} );
@@ -469,7 +470,7 @@ sub insert_rr_spf {
     my ( $self, $row_id, $spf ) = @_;
     my (@fields, @values);
     for ( qw/ domain scope result / ) {
-        next if ! $spf->{$_};
+        next if ! defined $spf->{$_};
         if ( 'domain' eq $_ ) {
             push @fields, 'domain_id';
             push @values, $self->get_domain_id( $spf->{domain} );
@@ -519,8 +520,12 @@ sub db_connect {
     my $user = $self->config->{report_store}{user};
     my $pass = $self->config->{report_store}{pass};
 
-    if ($self->{grammar} and $self->{grammar}->dsn =~ /$dsn/i) {
-        return $self->{dbix} if $self->{dbix};    # caching
+    # cacheing
+    if ($self->{grammar} && $self->{dbix}) {
+        my $cached_grammar_type = $self->{grammar}->dsn;
+        if ( $dsn =~ /$cached_grammar_type/ ) {
+            return $self->{dbix};    # caching
+        }
     }
 
     my $needs_tables;
@@ -549,6 +554,7 @@ sub db_connect {
     if ($needs_tables) {
         $self->apply_db_schema($needs_tables);
     }
+
     return $self->{dbix};
 }
 
@@ -597,12 +603,9 @@ sub query {
 
     my $dbix = $self->db_connect() or croak DBIx::Simple->error;
 
-    return $self->query_insert( $query, $err, @params )
-        if $query =~ /^INSERT/ix;
-    return $self->query_replace( $query, $err, @params )
-        if $query =~ /^(?:REPLACE|UPDATE)/ix;
-    return $self->query_delete( $query, $err, @params )
-        if $query =~ /^(?:DELETE|TRUNCATE)/ix;
+    return $self->query_insert( $query, $err, @params )  if $query =~ /^INSERT/ix;
+    return $self->query_replace( $query, $err, @params ) if $query =~ /^(?:REPLACE|UPDATE)/ix;
+    return $self->query_delete( $query, $err, @params )  if $query =~ /^(?:DELETE|TRUNCATE)/ix;
     return $self->query_any( $query, $err, @params );
 }
 
@@ -646,7 +649,6 @@ sub query_delete {
     return $affected;
 }
 
-
 sub get_row_spf {
     my ($self, $rowid) = @_;
     return $self->query( $self->grammar->select_row_spf, [ $rowid ] );
@@ -680,7 +682,7 @@ Mail::DMARC::Report::Store::SQL - store and retrieve reports from a SQL RDBMS
 
 =head1 VERSION
 
-version 1.20191025
+version 1.20240314
 
 =head1 DESCRIPTION
 
@@ -714,7 +716,7 @@ Marc Bradshaw <marc@marcbradshaw.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019 by Matt Simerson.
+This software is copyright (c) 2024 by Matt Simerson.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
